@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2007 Henrique Pinto <henrique.pinto@kdemail.net>
  * Copyright (c) 2008-2009 Harald Hvaal <haraldhv@stud.ntnu.no>
- * Copyright (c) 2009 Raphael Kubo da Costa <kubito@gmail.com>
+ * Copyright (c) 2009-2010 Raphael Kubo da Costa <kubito@gmail.com>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,9 +41,10 @@
 
 namespace Kerfuffle
 {
+
 class ThreadExecution;
 
-class KERFUFFLE_EXPORT Job: public KJob, public ArchiveObserver
+class KERFUFFLE_EXPORT Job : public KJob, public ArchiveObserver
 {
     Q_OBJECT
 
@@ -76,21 +77,23 @@ signals:
     void entryRemoved(const QString & entry);
     void error(const QString& errorMessage, const QString& details);
     void newEntry(const ArchiveEntry &);
-    void userQuery(Query*);
+    void userQuery(Kerfuffle::Query*);
 };
 
-class KERFUFFLE_EXPORT ListJob: public Job
+class KERFUFFLE_EXPORT ListJob : public Job
 {
     Q_OBJECT
 
 public:
     explicit ListJob(ReadOnlyArchiveInterface *interface, QObject *parent = 0);
 
-    void doWork();
-    qlonglong extractedFilesSize();
-    bool isPasswordProtected();
-    bool isSingleFolderArchive();
-    QString subfolderName();
+    qlonglong extractedFilesSize() const;
+    bool isPasswordProtected() const;
+    bool isSingleFolderArchive() const;
+    QString subfolderName() const;
+
+public slots:
+    virtual void doWork();
 
 private:
     bool m_isSingleFolderArchive;
@@ -103,52 +106,54 @@ private slots:
     void onNewEntry(const ArchiveEntry&);
 };
 
-class KERFUFFLE_EXPORT ExtractJob: public Job
+class KERFUFFLE_EXPORT ExtractJob : public Job
 {
     Q_OBJECT
 
 public:
-    ExtractJob(const QList<QVariant> & files, const QString&
-               destinationDir, ExtractionOptions options,
-               ReadOnlyArchiveInterface *interface, QObject *parent = 0);
+    ExtractJob(const QVariantList& files, const QString& destinationDir, ExtractionOptions options, ReadOnlyArchiveInterface *interface, QObject *parent = 0);
 
-    void doWork();
+public slots:
+    virtual void doWork();
 
 private:
+    // TODO: Maybe this should be a method if ExtractionOptions were a class?
     void fillInDefaultValues(ExtractionOptions& options);
-    QList<QVariant> m_files;
+
+    QVariantList m_files;
     QString m_destinationDir;
     ExtractionOptions m_options;
 };
 
-class KERFUFFLE_EXPORT AddJob: public Job
+class KERFUFFLE_EXPORT AddJob : public Job
 {
     Q_OBJECT
 
 public:
-    AddJob(const QStringList & files, const CompressionOptions& options,
-           ReadWriteArchiveInterface *interface, QObject *parent = 0
-          );
-    void doWork();
+    AddJob(const QStringList& files, const CompressionOptions& options, ReadWriteArchiveInterface *interface, QObject *parent = 0);
+
+public slots:
+    virtual void doWork();
 
 private:
     QStringList m_files;
     CompressionOptions m_options;
-
 };
 
-class KERFUFFLE_EXPORT DeleteJob: public Job
+class KERFUFFLE_EXPORT DeleteJob : public Job
 {
     Q_OBJECT
 
 public:
-    DeleteJob(const QList<QVariant>& files, ReadWriteArchiveInterface
-              *interface, QObject *parent = 0);
-    void doWork();
+    DeleteJob(const QVariantList& files, ReadWriteArchiveInterface *interface, QObject *parent = 0);
+
+public slots:
+    virtual void doWork();
 
 private:
-    QList<QVariant> m_files;
+    QVariantList m_files;
 };
+
 } // namespace Kerfuffle
 
 #endif // JOBS_H
