@@ -135,13 +135,12 @@ Part::~Part()
 void Part::registerJob(KJob* job)
 {
     if (!m_jobTracker) {
-        m_jobTracker = new JobTracker();
+        m_jobTracker = new JobTracker(widget());
         m_statusBarExtension->addStatusBarItem(m_jobTracker->widget(0), 0, true);
         m_jobTracker->widget(job)->show();
     }
     m_jobTracker->registerJob(job);
 
-    //KIO::getJobTracker()->registerJob(job);
     emit busy();
     connect(job, SIGNAL(result(KJob*)), this, SIGNAL(ready()));
 }
