@@ -14,6 +14,7 @@
 #include <QFont>
 #include <QProcess>
 
+#include <KActionCollection>
 #include <KMessageBox>
 #include <KPassivePopup>
 #include <KTar>
@@ -31,9 +32,10 @@
 #include "kgpglibrary.h"
 #include "kgpgsettings.h"
 #include "kgpgtextinterface.h"
-#include "kgpgview.h"
+#include "kgpgtextedit.h"
 #include "selectpublickeydialog.h"
 #include "selectsecretkey.h"
+#include "kgpginterface.h"
 
 KGpgExternalActions::KGpgExternalActions(KeysManager *parent, KGpgItemModel *model)
 	: QObject(parent),
@@ -58,11 +60,11 @@ void KGpgExternalActions::encryptDroppedFile()
 
 	if (KGpgSettings::encryptFilesTo()) {
 		if (KGpgSettings::pgpCompatibility())
-			opts << QString::fromLocal8Bit("--pgp6");
+			opts << "--pgp6";
 		
-		lib->slotFileEnc(droppedUrls, opts, m_model, goDefaultKey, KGpgSettings::fileEncryptionKey());
+		lib->slotFileEnc(droppedUrls, opts, m_model, goDefaultKey(), KGpgSettings::fileEncryptionKey());
 	} else {
-		lib->slotFileEnc(droppedUrls, opts, m_model, goDefaultKey);
+		lib->slotFileEnc(droppedUrls, opts, m_model, goDefaultKey());
 	}
 }
 
@@ -72,13 +74,13 @@ void KGpgExternalActions::encryptDroppedFolder()
 	kgpgfoldertmp = new KTemporaryFile();
 	kgpgfoldertmp->open();
 
-	if (KMessageBox::Cancel == KMessageBox::warningContinueCancel(0,
+	if (KMessageBox::Cancel == KMessageBox::warningContinueCancel(m_keysmanager,
 				i18n("<qt>KGpg will now create a temporary archive file:<br /><b>%1</b> to process the encryption. The file will be deleted after the encryption is finished.</qt>",
 				kgpgfoldertmp->fileName()), i18n("Temporary File Creation"), KStandardGuiItem::cont(),
 				KStandardGuiItem::cancel(), "FolderTmpFile"))
 		return;
 
-	dialog = new KgpgSelectPublicKeyDlg(0, m_model, goDefaultKey, droppedUrls.first().fileName());
+	dialog = new KgpgSelectPublicKeyDlg(m_keysmanager, m_model, goDefaultKey(), false, droppedUrls);
 
 	KHBox *bGroup = new KHBox(dialog->optionsbox);
 
@@ -89,6 +91,7 @@ void KGpgExternalActions::encryptDroppedFolder()
 	optionbx->addItem(i18n("Gzip"));
 	optionbx->addItem(i18n("Bzip2"));
 	optionbx->addItem(i18n("Tar"));
+	optionbx->addItem(i18n("Tar/XZ"));
 
 	connect(optionbx, SIGNAL(activated (int)), SLOT(slotSetCompression(int)));
 	connect(dialog, SIGNAL(okClicked()), SLOT(startFolderEncode()));
@@ -99,6 +102,7 @@ void KGpgExternalActions::encryptDroppedFolder()
 
 void KGpgExternalActions::slotAbortEnc()
 {
+	dialog->deleteLater();
 	dialog = NULL;
 }
 
@@ -127,8 +131,11 @@ void KGpgExternalActions::startFolderEncode()
 	case 3:
 		extension = ".tar";
 		break;
+	case 4:
+		extension = ".tar.xz";
+		break;
 	default:
-		Q_ASSERT(compressionScheme);
+		Q_ASSERT(compressionScheme == 0);
 		return;
 	}
 
@@ -152,12 +159,15 @@ void KGpgExternalActions::startFolderEncode()
 	QFile encryptedFolder(encryptedFile.path());
 	if (encryptedFolder.exists()) {
 		dialog->hide();
-		KIO::RenameDialog over(0, i18n("File Already Exists"), KUrl(), encryptedFile, KIO::M_OVERWRITE);
-		if (over.exec() == QDialog::Rejected) {
+		QPointer<KIO::RenameDialog> over = new KIO::RenameDialog(m_keysmanager, i18n("File Already Exists"), KUrl(), encryptedFile, KIO::M_OVERWRITE);
+		if (over->exec() == QDialog::Rejected) {
+			dialog->deleteLater();
 			dialog = NULL;
+			delete over;
 			return;
 		}
-		encryptedFile = over.newDestUrl();
+		encryptedFile = over->newDestUrl();
+		delete over;
 		dialog->show(); // strange, but if dialog is hidden, the passive popup is not displayed...
 	}
 
@@ -167,6 +177,7 @@ void KGpgExternalActions::startFolderEncode()
 	pop->show();
 	kapp->processEvents();
 	dialog->accept();
+	dialog->deleteLater();
 	dialog = NULL;
 
 	KArchive *arch = NULL;
@@ -183,8 +194,11 @@ void KGpgExternalActions::startFolderEncode()
 	case 3:
 		arch = new KTar(kgpgfoldertmp->fileName(), "application/x-tar");
 		break;
+	case 4:
+		arch = new KTar(kgpgfoldertmp->fileName(), "application/x-xz");
+		break;
 	default:
-		Q_ASSERT(1);
+		Q_ASSERT(0);
 	}
 
 	if (!arch->open(QIODevice::WriteOnly))
@@ -198,24 +212,24 @@ void KGpgExternalActions::startFolderEncode()
 	arch->close();
 	delete arch;
 
-	KGpgTextInterface *folderprocess = new KGpgTextInterface();
-	connect(folderprocess, SIGNAL(fileEncryptionFinished(KUrl, KGpgTextInterface*)), SLOT(slotFolderFinished(KUrl, KGpgTextInterface*)));
-	connect(folderprocess, SIGNAL(errorMessage(const QString &, KGpgTextInterface*)), SLOT(slotFolderFinishedError(const QString &, KGpgTextInterface*)));
+	KGpgTextInterface *folderprocess = new KGpgTextInterface(this);
+	connect(folderprocess, SIGNAL(fileEncryptionFinished(KUrl)), SLOT(slotFolderFinished(KUrl)));
+	connect(folderprocess, SIGNAL(errorMessage(const QString &)), SLOT(slotFolderFinishedError(const QString &)));
 	folderprocess->encryptFile(selec, KUrl(kgpgfoldertmp->fileName()), encryptedFile, encryptOptions, symetric);
 }
 
-void KGpgExternalActions::slotFolderFinished(const KUrl &, KGpgTextInterface *iface)
+void KGpgExternalActions::slotFolderFinished(const KUrl &)
 {
 	delete pop;
 	delete kgpgfoldertmp;
-	iface->deleteLater();
+	sender()->deleteLater();
 }
 
-void KGpgExternalActions::slotFolderFinishedError(const QString &errmsge, KGpgTextInterface *iface)
+void KGpgExternalActions::slotFolderFinishedError(const QString &errmsge)
 {
 	delete pop;
 	delete kgpgfoldertmp;
-	iface->deleteLater();
+	sender()->deleteLater();
 	KMessageBox::sorry(0, errmsge);
 }
 
@@ -252,7 +266,7 @@ void KGpgExternalActions::slotVerifyFile()
 
 	QString sigfile;
 	// try to find detached signature.
-	if (!droppedUrl.fileName().endsWith(".sig")) {
+	if (!droppedUrl.fileName().endsWith(QLatin1String(".sig"))) {
 		sigfile = droppedUrl.path() + ".sig";
 		QFile fsig(sigfile);
 		if (!fsig.exists()) {
@@ -268,7 +282,7 @@ void KGpgExternalActions::slotVerifyFile()
 	}
 
 	// pipe gpg command
-	KGpgTextInterface *verifyFileProcess = new KGpgTextInterface();
+	KGpgTextInterface *verifyFileProcess = new KGpgTextInterface(this);
 	connect (verifyFileProcess, SIGNAL(verifyquerykey(QString)), SLOT(importSignature(QString)));
 	verifyFileProcess->KgpgVerifyFile(droppedUrl, KUrl(sigfile));
 }
@@ -287,7 +301,7 @@ void KGpgExternalActions::signDroppedFile()
 		return;
 
 	// select a private key to sign file --> listkeys.cpp
-	KgpgSelectSecretKey *opts = new KgpgSelectSecretKey(0, m_model, false);
+	QPointer<KgpgSelectSecretKey> opts = new KgpgSelectSecretKey(0, m_model, false);
 	if (opts->exec() != QDialog::Accepted) {
 		delete opts;
 		return;
@@ -302,9 +316,8 @@ void KGpgExternalActions::signDroppedFile()
 		Options << "--pgp6";
 	Options << "--detach-sign";
 
-	KGpgTextInterface *signFileProcess = new KGpgTextInterface();
-	signFileProcess->signFilesBlocking(signKeyID, droppedUrls, Options);
-	delete signFileProcess;
+	KGpgTextInterface signFileProcess;
+	signFileProcess.signFilesBlocking(signKeyID, droppedUrls, Options);
 }
 
 void KGpgExternalActions::decryptDroppedFile()
@@ -318,11 +331,13 @@ void KGpgExternalActions::decryptFile(KgpgLibrary *lib)
 {
 	if (!droppedUrls.first().isLocalFile()) {
 		showDroppedFile();
-		decryptNextFile(lib, KUrl());
+		decryptNextFile(KUrl(), lib);
 	}
 	
 	QString oldname(droppedUrls.first().fileName());
-	if (oldname.endsWith(".gpg", Qt::CaseInsensitive) || oldname.endsWith(".asc", Qt::CaseInsensitive) || oldname.endsWith(".pgp", Qt::CaseInsensitive))
+	if (oldname.endsWith(QLatin1String(".gpg"), Qt::CaseInsensitive) ||
+			oldname.endsWith(QLatin1String(".asc"), Qt::CaseInsensitive) ||
+			oldname.endsWith(QLatin1String(".pgp"), Qt::CaseInsensitive))
 		oldname.chop(4);
 	else
 		oldname.append(".clear");
@@ -330,25 +345,30 @@ void KGpgExternalActions::decryptFile(KgpgLibrary *lib)
 	KUrl swapname(droppedUrls.first().directory(KUrl::AppendTrailingSlash) + oldname);
 	QFile fgpg(swapname.path());
 	if (fgpg.exists()) {
-		KIO::RenameDialog over(0, i18n("File Already Exists"), KUrl(), swapname, KIO::M_OVERWRITE);
-		if (over.exec() == QDialog::Rejected) {
-			decryptNextFile(lib, KUrl());
+		QPointer<KIO::RenameDialog> over = new KIO::RenameDialog(m_keysmanager, i18n("File Already Exists"), KUrl(), swapname, KIO::M_OVERWRITE);
+		if (over->exec() != QDialog::Accepted) {
+			delete over;
+			decryptNextFile(KUrl(), lib);
 			return;
 		}
 
-		swapname=over.newDestUrl();
+		swapname = over->newDestUrl();
+		delete over;
 	}
 
 	QStringList custdecr;
 	if (!KGpgSettings::customDecrypt().isEmpty())
 		custdecr.append(KGpgSettings::customDecrypt());
 	connect(lib, SIGNAL(systemMessage(QString, bool)), SLOT(busyMessage(QString, bool)));
-	connect(lib, SIGNAL(decryptionOver(KgpgLibrary *, KUrl)), SLOT(decryptNextFile(KgpgLibrary *, KUrl)));
+	connect(lib, SIGNAL(decryptionOver(KUrl)), SLOT(decryptNextFile(KUrl)));
 	lib->slotFileDec(droppedUrls.first(), swapname, custdecr);
 }
 
-void KGpgExternalActions::decryptNextFile(KgpgLibrary *lib, const KUrl &failed)
+void KGpgExternalActions::decryptNextFile(const KUrl &failed, KgpgLibrary *lib)
 {
+	if (lib == NULL)
+		lib = qobject_cast<KgpgLibrary *>(sender());
+
 	if (!failed.isEmpty())
 		m_decryptionFailed << failed;
 
@@ -369,8 +389,7 @@ void KGpgExternalActions::decryptNextFile(KgpgLibrary *lib, const KUrl &failed)
 void KGpgExternalActions::showDroppedFile()
 {
 	KgpgEditor *kgpgtxtedit = new KgpgEditor(m_keysmanager, m_model, 0);
-	kgpgtxtedit->setAttribute(Qt::WA_DeleteOnClose);
-	kgpgtxtedit->view->editor->slotDroppedFile(droppedUrls.first());
+	kgpgtxtedit->m_editor->slotDroppedFile(droppedUrls.first());
 
 	connect(kgpgtxtedit, SIGNAL(encryptFiles(KUrl::List)), SLOT(encryptFiles(KUrl::List)));
 	connect(m_keysmanager, SIGNAL(fontChanged(QFont)), kgpgtxtedit, SLOT(slotSetFont(QFont)));
@@ -406,7 +425,7 @@ void KGpgExternalActions::firstRun()
 	QProcess *createConfigProc = new QProcess(this);
 	QStringList args;
 	args << "--no-tty" << "--list-secret-keys";
-	createConfigProc->start("gpg", args);	// start gnupg so that it will create a config file
+	createConfigProc->start("gpg", args);	// start GnuPG so that it will create a config file
 	createConfigProc->waitForFinished();
 	startAssistant();
 }
@@ -454,3 +473,8 @@ void KGpgExternalActions::help()
 {
 	KToolInvocation::invokeHelp(0, "kgpg");
 }
+
+KShortcut KGpgExternalActions::goDefaultKey() const
+{
+	return qobject_cast<KAction *>(m_keysmanager->actionCollection()->action("go_default_key"))->shortcut();
+}
