@@ -48,33 +48,6 @@ KgpgInterface::~KgpgInterface()
 {
 }
 
-QString KgpgInterface::checkForUtf8(QString txt)
-{
-	/* The string is not in UTF-8 */
-	if (!txt.contains("\\x"))
-		return txt;
-
-	// if (!strchr (txt.toAscii(), 0xc3) || (txt.find("\\x")!=-1)) {
-	for (int idx = 0; (idx = txt.indexOf( "\\x", idx )) >= 0 ; ++idx) {
-		char str[2] = { 'x', '\0'};
-		str[0] = (char) QString(txt.mid(idx + 2, 2)).toShort(0, 16);
-		txt.replace(idx, 4, str);
-	}
-
-	return QString::fromUtf8(txt.toAscii());
-}
-
-QString KgpgInterface::checkForUtf8bis(QString txt)
-{
-	if (strchr(txt.toAscii(), 0xc3) || txt.contains("\\x")) {
-		txt = checkForUtf8(txt);
-	} else {
-		txt = checkForUtf8(txt);
-		txt = QString::fromUtf8(txt.toAscii());
-	}
-	return txt;
-}
-
 int KgpgInterface::gpgVersion(const QString &vstr)
 {
 	if (vstr.isEmpty())
@@ -400,23 +373,6 @@ int KgpgInterface::sendPassphrase(const QString &text, KProcess *process, const
 	return 0;
 }
 
-void KgpgInterface::updateIDs(QString txt)
-{
-	int cut = txt.indexOf(' ', 22, Qt::CaseInsensitive);
-	txt.remove(0, cut);
-
-	if (txt.contains('(', Qt::CaseInsensitive))
-		txt = txt.section('(', 0, 0) + txt.section(')', -1);
-
-	txt.replace('<', "&lt;");
-
-	if (!userIDs.contains(txt)) {
-		if (!userIDs.isEmpty())
-			userIDs += i18n(" or ");
-		userIDs += txt;
-	}
-}
-
 KgpgKeyList KgpgInterface::readPublicKeys(const bool block, const QStringList &ids, const bool withsigs)
 {
 	m_publiclistkeys = KgpgKeyList();
@@ -435,8 +391,8 @@ KgpgKeyList KgpgInterface::readPublicKeys(const bool block, const QStringList &i
 	process->setOutputChannelMode(KProcess::MergedChannels);
 
 	if (!block) {
-		connect(process, SIGNAL(readReady(GPGProc *)), SLOT(readPublicKeysProcess(GPGProc *)));
-		connect(process, SIGNAL(processExited(GPGProc *)), SLOT(readPublicKeysFin(GPGProc *)));
+		connect(process, SIGNAL(readReady()), SLOT(readPublicKeysProcess()));
+		connect(process, SIGNAL(processExited()), SLOT(readPublicKeysFin()));
 		process->start();
 		return KgpgKeyList();
 	} else {
@@ -452,6 +408,8 @@ void KgpgInterface::readPublicKeysProcess(GPGProc *p)
 {
 	QStringList lsp;
 	int items;
+	if (p == NULL)
+		p = qobject_cast<GPGProc *>(sender());
 
 	while ((items = p->readln(lsp)) >= 0) {
 		if ((lsp.at(0) == "pub") && (items >= 10)) {
@@ -466,13 +424,13 @@ void KgpgInterface::readPublicKeysProcess(GPGProc *p)
 			m_publickey.setSize(lsp.at(2).toUInt());
 			m_publickey.setAlgorithm(Convert::toAlgo(lsp.at(3).toInt()));
 			m_publickey.setFingerprint(lsp.at(4));
-			m_publickey.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()).date());
+			m_publickey.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()));
 			m_publickey.setOwnerTrust(Convert::toOwnerTrust(lsp.at(8)));
 
 			if (lsp.at(6).isEmpty())
-				m_publickey.setExpiration(QDate());
+				m_publickey.setExpiration(QDateTime());
 			else
-				m_publickey.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()).date());
+				m_publickey.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()));
 
 			m_publickey.setValid((items <= 11) || !lsp.at(11).contains('D', Qt::CaseSensitive));  // disabled key
 
@@ -490,7 +448,7 @@ void KgpgInterface::readPublicKeysProcess(GPGProc *p)
 			sub.setTrust(Convert::toTrust(lsp.at(1)));
 			sub.setSize(lsp.at(2).toUInt());
 			sub.setAlgorithm(Convert::toAlgo(lsp.at(3).toInt()));
-			sub.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()).date());
+			sub.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()));
 
 			// FIXME: Please see kgpgkey.h, KgpgSubKey class
 			if (items <= 11) {
@@ -506,9 +464,9 @@ void KgpgInterface::readPublicKeysProcess(GPGProc *p)
 			}
 
 			if (lsp.at(6).isEmpty())
-				sub.setExpiration(QDate());
+				sub.setExpiration(QDateTime());
 			else
-				sub.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()).date());
+				sub.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()));
 
 			m_publickey.subList()->append(sub);
 			cycle = "sub";
@@ -516,7 +474,7 @@ void KgpgInterface::readPublicKeysProcess(GPGProc *p)
 			m_numberid++;
 			KgpgKeyUat uat;
 			uat.setId(QString::number(m_numberid));
-			uat.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()).date());
+			uat.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()));
 			m_publickey.uatList()->append(uat);
 
 			cycle = "uat";
@@ -571,12 +529,12 @@ void KgpgInterface::readPublicKeysProcess(GPGProc *p)
 			KgpgKeySign signature;
 
 			signature.setId(lsp.at(4));
-			signature.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()).date());
+			signature.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()));
 
 			if (lsp.at(6).isEmpty())
-				signature.setExpiration(QDate());
+				signature.setExpiration(QDateTime());
 			else
-				signature.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()).date());
+				signature.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()));
 
 			if (lsp.at(10).endsWith('l'))
 				signature.setLocal(true);
@@ -599,6 +557,9 @@ void KgpgInterface::readPublicKeysProcess(GPGProc *p)
 
 void KgpgInterface::readPublicKeysFin(GPGProc *p, const bool block)
 {
+	if (p == NULL)
+		p = qobject_cast<GPGProc *>(sender());
+
 	// insert the last key
 	if (cycle != "none")
 		m_publiclistkeys << m_publickey;
@@ -610,14 +571,12 @@ void KgpgInterface::readPublicKeysFin(GPGProc *p, const bool block)
 
 	p->deleteLater();
 	if (!block)
-		emit readPublicKeysFinished(m_publiclistkeys, this);
+		emit readPublicKeysFinished(m_publiclistkeys);
 }
 
 
 KgpgKeyList KgpgInterface::readSecretKeys(const QStringList &ids)
 {
-	m_partialline.clear();
-	m_ispartial = false;
 	m_secretlistkeys = KgpgKeyList();
 	m_secretkey = KgpgKey();
 	m_secretactivate = false;
@@ -657,13 +616,13 @@ void KgpgInterface::readSecretKeysProcess(GPGProc *p)
 			m_secretkey.setSize(lsp.at(2).toUInt());
 			m_secretkey.setAlgorithm(Convert::toAlgo(lsp.at(3).toInt()));
 			m_secretkey.setFingerprint(lsp.at(4));
-			m_secretkey.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()).date());
+			m_secretkey.setCreation(QDateTime::fromTime_t(lsp.at(5).toUInt()));
 			m_secretkey.setSecret(true);
 
 			if (lsp.at(6).isEmpty())
-				m_secretkey.setExpiration(QDate());
+				m_secretkey.setExpiration(QDateTime());
 			else
-				m_secretkey.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()).date());
+				m_secretkey.setExpiration(QDateTime::fromTime_t(lsp.at(6).toUInt()));
 			hasuid = true;
 		} else if ((lsp.at(0) == "uid") && (items >= 10)) {
 			if (hasuid)
@@ -745,18 +704,22 @@ QPixmap KgpgInterface::loadPhoto(const QString &keyid, const QString &uid, const
 	const QString pgpgoutput("echo %I");
 #endif
 
-	m_workProcess = new KProcess(this);
-	m_workProcess->setOutputChannelMode(KProcess::OnlyStdoutChannel);
-	*m_workProcess << KGpgSettings::gpgBinaryPath();
-	*m_workProcess << "--no-secmem-warning" << "--no-tty" << "--status-fd=2";
-	*m_workProcess << "--photo-viewer" << pgpgoutput << "--edit-key" << keyid << "uid" << uid << "showphoto" << "quit";
+	KProcess *workProcess = new KProcess(this);
+	workProcess->setOutputChannelMode(KProcess::OnlyStdoutChannel);
+	*workProcess << KGpgSettings::gpgBinaryPath();
+	*workProcess << "--no-secmem-warning" << "--no-greeting" << "--no-tty" << "--status-fd=2";
+	*workProcess << "--photo-viewer" << pgpgoutput << "--edit-key" << keyid << "uid" << uid << "showphoto" << "quit";
 
-	connect(m_workProcess, SIGNAL(finished(int)), SLOT(loadPhotoFin(int)));
-	m_workProcess->start();
+	if (!block)
+		connect(workProcess, SIGNAL(finished(int)), SLOT(loadPhotoFin(int)));
+	workProcess->start();
 	if (!block) {
 		return QPixmap();
 	} else {
-		m_workProcess->waitForFinished();
+		workProcess->waitForFinished();
+		if (workProcess->exitCode() == 0)
+			readPixmapFromProcess(workProcess);
+		delete workProcess;
 		return m_pixmap;
 	}
 }
@@ -766,171 +729,26 @@ void KgpgInterface::loadPhotoFin(int exitCode)
 	m_pixmap = QPixmap();
 
 	if (exitCode == 0) {
-		QByteArray pic(m_workProcess->readAllStandardOutput());
-
-		while (pic.endsWith('\r') || pic.endsWith('\n'))
-			pic.chop(1);
-
-		KUrl url(pic);
-		m_pixmap.load(url.path());
-		QFile::remove(url.path());
-		QDir dir;
-		dir.rmdir(url.directory());
+		readPixmapFromProcess(qobject_cast<KProcess *>(sender()));
 	}
 
-	emit loadPhotoFinished(m_pixmap, this);
+	sender()->deleteLater();
+
+	emit loadPhotoFinished(m_pixmap);
 }
 
-// delete signature
-void KgpgInterface::KgpgDelSignature(const QString &keyID, const QString &uid, QString signKeyID)
+void KgpgInterface::readPixmapFromProcess(KProcess *proc)
 {
-	deleteSuccess = false;
-	step = 0;
+	QByteArray pic(proc->readAllStandardOutput());
 
-	signb = 0;
-	sigsearch = 0;
-	QList<int> signoff;
+	while (pic.endsWith('\r') || pic.endsWith('\n'))
+		pic.chop(1);
 
-	findSigns(keyID, QStringList(signKeyID), uid, &signoff);
-
-	if (signoff.count() == 0)
-		return;
-
-	signb = signoff.at(0);
-
-	GPGProc *conprocess = new GPGProc(this);
-	*conprocess << "--command-fd=0" << "--status-fd=1" << "--edit-key" << keyID << "uid" << uid << "delsig";
-	connect(conprocess, SIGNAL(readReady(GPGProc *)), SLOT(delsigprocess(GPGProc *)));
-	connect(conprocess, SIGNAL(processExited(GPGProc *)), SLOT(delsignover(GPGProc *)));
-	conprocess->start();
-}
-
-void KgpgInterface::delsigprocess(GPGProc *p)
-{
-	QString required;
-
-	while (p->readln(required, true) >= 0) {
-		if (required.contains("keyedit.delsig")){
-			if ((sigsearch == signb) && (step == 0)) {
-				p->write("Y\n");
-				step = 1;
-			} else
-				p->write("n\n");
-			sigsearch++;
-			required.clear();
-		} else if ((step == 1) && required.contains("keyedit.prompt")) {
-			p->write("save\n");
-			required.clear();
-			deleteSuccess = true;
-		} else if (required.contains("GET_LINE")) {
-			p->write("quit\n");
-			deleteSuccess = false;
-		}
-	}
-}
-
-void KgpgInterface::delsignover(GPGProc *p)
-{
-	p->deleteLater();
-	emit delsigfinished(deleteSuccess);
-}
-
-/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////  key revocation
-
-void KgpgInterface::KgpgRevokeKey(const QString &keyID, const KUrl &revokeUrl, const int reason, const QString &description)
-{
-	revokeReason = reason;
-	revokeSuccess = false;
-	revokeDescription = description;
-	certificateUrl = revokeUrl;
-	output.clear();
-
-	GPGProc *process = new GPGProc(this);
-	*process << "--status-fd=1" << "--command-fd=0";
-
-	if (!revokeUrl.isEmpty())
-		*process << "-o" << revokeUrl.toLocalFile();
-	*process << "--gen-revoke" << keyID;
-	QObject::connect(process, SIGNAL(processExited(GPGProc *)), SLOT(revokeover(GPGProc *)));
-	QObject::connect(process, SIGNAL(readReady(GPGProc *)), SLOT(revokeprocess(GPGProc *)));
-	process->start();
-}
-
-void KgpgInterface::revokeover(GPGProc *)
-{
-	if (!revokeSuccess) {
-		KMessageBox::detailedSorry(0,i18n("Creation of the revocation certificate failed..."), output);
-	} else {
-		output = output.section("-----BEGIN", 1);
-		output.prepend("-----BEGIN");
-		output = output.section("BLOCK-----", 0);
-		emit revokecertificate(output);
-		if (!certificateUrl.isEmpty())
-			emit revokeurl(certificateUrl);
-	}
-}
-
-void KgpgInterface::revokeprocess(GPGProc *p)
-{
-	QString required;
-
-	while (p->readln(required) >= 0) {
-		output += required + '\n';
-
-		if (required.contains("USERID_HINT",Qt::CaseInsensitive)) {
-			updateIDs(required);
-
-		} else if (required.contains("GOOD_PASSPHRASE")) {
-			revokeSuccess=true;
-
-		} else if (required.contains("gen_revoke.okay") ||
-					required.contains("ask_revocation_reason.okay") ||
-					required.contains("openfile.overwrite.okay")) {
-			p->write("YES\n");
-		} else if (required.contains("ask_revocation_reason.code")) {
-			p->write(QString::number(revokeReason).toAscii() + '\n');
-		} else if (required.contains("passphrase.enter")) {
-			if (sendPassphrase(i18n("<qt>Enter passphrase for <b>%1</b>:</qt>", checkForUtf8bis(userIDs)), p)) {
-				expSuccess=3;  //  aborted by user mode
-				p->write("quit\n");
-				return;
-			}
-		} else if (required.contains("ask_revocation_reason.text")) {
-			p->write(revokeDescription.toAscii() + '\n');
-			revokeDescription.clear();
-		} else if ((required.contains("GET_"))) {
-			// gpg asks for something unusal, turn to konsole mode
-			kDebug(2100) << "unknown request:" << required;
-			expSuccess=1;  /////  switching to console mode
-			p->write("quit\n");
-		}
-	}
-}
-
-void KgpgInterface::findSigns(const QString &keyID, const QStringList &ids, const QString &uid, QList<int> *res)
-{
-	GPGProc *listproc = new GPGProc(this);
-	*listproc << "--status-fd=1";
-	*listproc << "--with-colons" << "--list-sigs" << "--fixed-list-mode" << keyID;
-	listproc->start();
-	listproc->waitForFinished(-1);
-
-	QString line;
-	int curuid = 0;
-	int signs = 0;
-	int tgtuid = uid.toInt();
-
-	while (listproc->readln(line, true) >= 0) {
-		if (line.startsWith(QLatin1String("sig:")) && (tgtuid == curuid)) {
-			if (ids.contains(line.section(':', 4, 4), Qt::CaseInsensitive))
-				*res << signs;
-			signs++;
-		} else if (line.startsWith(QLatin1String("uid:"))) {
-			curuid++;
-		}
-	}
-
-	delete listproc;
+	KUrl url(pic);
+	m_pixmap.load(url.path());
+	QFile::remove(url.path());
+	QDir dir;
+	dir.rmdir(url.directory());
 }
 
 #include "kgpginterface.moc"
