@@ -21,14 +21,19 @@
 #ifndef ALLYOURBASE_H
 #define ALLYOURBASE_H
 
-#include <kiconview.h>
-#include <klistview.h>
+#include <k3iconview.h>
+#include <k3listview.h>
 #include <kwallet.h>
 #include <kiconloader.h>
 #include <kicontheme.h>
+//Added by qt3to4:
+#include <QPixmap>
+#include <QDropEvent>
+#include <QDragEnterEvent>
+#include <QMouseEvent>
 
-#define KWALLETENTRYMAGIC ((Q_UINT32) 0x6B776C65)
-#define KWALLETFOLDERMAGIC ((Q_UINT32) 0x6B776C66)
+#define KWALLETENTRYMAGIC ((quint32) 0x6B776C65)
+#define KWALLETFOLDERMAGIC ((quint32) 0x6B776C66)
 
 enum KWalletListItemClasses {
 	KWalletFolderItemClass = 1000,
@@ -37,9 +42,9 @@ enum KWalletListItemClasses {
 	KWalletUnknownClass = 2000
 };
 
-class KWalletEntryItem : public KListViewItem {
+class KWalletEntryItem : public K3ListViewItem {
 	public:
-		KWalletEntryItem(KWallet::Wallet *w, QListViewItem* parent, const QString& ename);
+		KWalletEntryItem(KWallet::Wallet *w, Q3ListViewItem* parent, const QString& ename);
 		virtual ~KWalletEntryItem();
 
 		const QString& oldName() { return _oldName; }
@@ -55,9 +60,9 @@ class KWalletEntryItem : public KListViewItem {
 		QString _oldName;
 };
 
-class KWalletContainerItem : public KListViewItem {
+class KWalletContainerItem : public K3ListViewItem {
 	public:
-		KWalletContainerItem(QListViewItem* parent, const QString& name,
+		KWalletContainerItem(Q3ListViewItem* parent, const QString& name,
 		    KWallet::Wallet::EntryType type);
 		virtual ~KWalletContainerItem();
 
@@ -65,15 +70,15 @@ class KWalletContainerItem : public KListViewItem {
 		virtual int rtti() const;
 		KWallet::Wallet::EntryType type();
 		bool contains(const QString& itemKey);
-		QListViewItem* getItem(const QString& itemKey);
+		Q3ListViewItem* getItem(const QString& itemKey);
 
 	private:
 		KWallet::Wallet::EntryType _type;
 };
 
-class KWalletFolderItem : public KListViewItem {
+class KWalletFolderItem : public K3ListViewItem {
 	public:
-		KWalletFolderItem(KWallet::Wallet *w, QListView* parent, 
+		KWalletFolderItem(KWallet::Wallet *w, Q3ListView* parent, 
 			const QString& name, int entries);
 		virtual ~KWalletFolderItem();
 
@@ -83,9 +88,9 @@ class KWalletFolderItem : public KListViewItem {
 		QString name() const;
 		void refresh();
 		KWalletContainerItem* getContainer(KWallet::Wallet::EntryType type);
-		QPixmap getFolderIcon(KIcon::Group group);
+		QPixmap getFolderIcon(KIconLoader::Group group);
 		bool contains(const QString& itemKey);
-		QListViewItem* getItem(const QString& itemKey);
+		Q3ListViewItem* getItem(const QString& itemKey);
 
 	public:
 		KWallet::Wallet *_wallet;
@@ -95,10 +100,11 @@ class KWalletFolderItem : public KListViewItem {
 		int _entries;
 };
 
-class KWalletEntryList : public KListView {
+class KWalletEntryList : public K3ListView {
 	Q_OBJECT
+
 	public:
-		KWalletEntryList(QWidget *parent, const char *name = 0L);
+		explicit KWalletEntryList(QWidget *parent, const char *name = 0L);
 		virtual ~KWalletEntryList();
 
 		bool existsFolder(const QString& name);
@@ -108,40 +114,46 @@ class KWalletEntryList : public KListView {
 		void setWallet(KWallet::Wallet *w);
 
 	protected:
-		void itemDropped(QDropEvent *e, QListViewItem *item);
-		virtual QDragObject *dragObject();
+		void itemDropped(QDropEvent *e, Q3ListViewItem *item);
+		virtual Q3DragObject *dragObject();
 		virtual bool acceptDrag (QDropEvent* event) const;
 
 	private:
-		static KWalletFolderItem *getItemFolder(QListViewItem *item);
+		static KWalletFolderItem *getItemFolder(Q3ListViewItem *item);
 	
 	public:
 		KWallet::Wallet *_wallet;
 };
 
-class KWalletItem : public QIconViewItem {
+class KWalletItem : public Q3IconViewItem {
 	public:
-		KWalletItem(QIconView *parent, const QString& walletName);
+		KWalletItem(Q3IconView *parent, const QString& walletName);
 		virtual ~KWalletItem();
 
 		virtual bool acceptDrop(const QMimeSource *mime) const;
+		
+		void setOpen(bool state);
 
 	protected:
-		virtual void dropped(QDropEvent *e, const QValueList<QIconDragItem>& lst); 
+		virtual void dropped(QDropEvent *e, const Q3ValueList<Q3IconDragItem>& lst); 
+		
+	private:
+		bool _open;
 };
 
 
-class KWalletIconView : public KIconView {
+class KWalletIconView : public K3IconView {
 	Q_OBJECT
+
 	public:
-		KWalletIconView(QWidget *parent, const char *name = 0L);
+		explicit KWalletIconView(QWidget *parent, const char *name = 0L);
 		virtual ~KWalletIconView();
 
 	protected slots:
-		virtual void slotDropped(QDropEvent *e, const QValueList<QIconDragItem>& lst);
+		virtual void slotDropped(QDropEvent *e, const Q3ValueList<Q3IconDragItem>& lst);
 
 	protected:
-		virtual QDragObject *dragObject();
+		virtual Q3DragObject *dragObject();
 		virtual void contentsMousePressEvent(QMouseEvent *e);
 		QPoint _mousePos;
 };
@@ -151,7 +163,7 @@ inline QDataStream& operator<<(QDataStream& str, const KWalletEntryItem& w) {
 	QString name = w.text(0);
 	str << name;
 	KWallet::Wallet::EntryType et = w._wallet->entryType(name);
-	str << long(et);
+	str << qint64(et);
 	QByteArray a;
 	w._wallet->readEntry(name, a);
 	str << a;
@@ -166,7 +178,7 @@ inline QDataStream& operator<<(QDataStream& str, const KWalletFolderItem& w) {
 	for (QStringList::Iterator it = entries.begin(); it != entries.end(); ++it) {
 		str << *it;
 		KWallet::Wallet::EntryType et = w._wallet->entryType(*it);
-		str << long(et);
+		str << (qint32)et;
 		QByteArray a;
 		w._wallet->readEntry(*it, a);
 		str << a;
