@@ -1,7 +1,7 @@
 /*
     This file is part of the Okteta Gui library, part of the KDE project.
 
-    Copyright 2003,2007,2008 Friedrich W. H. Kossebau <kossebau@kde.org>
+    Copyright 2003,2007-2009 Friedrich W. H. Kossebau <kossebau@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -20,36 +20,35 @@
     License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef KHE_UI_ABSTRACTBYTEARRAYCOLUMNRENDERER_H
-#define KHE_UI_ABSTRACTBYTEARRAYCOLUMNRENDERER_H
+#ifndef OKTETA_ABSTRACTBYTEARRAYCOLUMNRENDERER_H
+#define OKTETA_ABSTRACTBYTEARRAYCOLUMNRENDERER_H
 
 // ColumnsView
-#include <columnrenderer.h>
+#include "abstractcolumnrenderer.h"
+#include "linepositionrange.h"
+#include "linerange.h"
 // Okteta core
-#include <khe.h>
-#include <abstractbytearraymodel.h>
-#include <khechar.h>
-// commonlib
-#include <section.h>
-
+#include "oktetacore.h"
+#include "abstractbytearraymodel.h"
+#include "character.h"
 
 class QPainter;
 class QColor;
 class QBrush;
+class QRect;
 
-namespace KHECore {
+
+namespace Okteta
+{
+class Coord;
 class Bookmarkable;
 class CharCodec;
-}
-
-namespace KHEUI
-{
 
 // class KByteArrayView;
 class ByteArrayTableRanges;
 class ByteArrayTableLayout;
 
-const int NoByteFound = -1;
+const LinePosition NoByteFound = -1;
 
 /** base class of all buffer column displayers
   * holds all information about the vertical layout of a buffer column
@@ -58,39 +57,39 @@ const int NoByteFound = -1;
   *
   *@author Friedrich W. H. Kossebau
   */
-class AbstractByteArrayColumnRenderer : public ColumnRenderer
+class OKTETAGUI_EXPORT AbstractByteArrayColumnRenderer : public AbstractColumnRenderer
 {
   public:
     enum FrameStyle { Frame, Left, Right };
   public:
-    AbstractByteArrayColumnRenderer( ColumnsView/*KByteArrayView*/ *columnsView,
-        KHECore::AbstractByteArrayModel* byteArrayModel, ByteArrayTableLayout* layout, ByteArrayTableRanges* ranges );
+    AbstractByteArrayColumnRenderer( AbstractColumnStylist* stylist,
+        AbstractByteArrayModel* byteArrayModel, ByteArrayTableLayout* layout, ByteArrayTableRanges* ranges );
     virtual ~AbstractByteArrayColumnRenderer();
 
 
-  public: // ColumnRenderer API
-    virtual void renderFirstLine( QPainter *painter, const KPixelXs &Xs, int firstLineIndex );
-    virtual void renderNextLine( QPainter *painter );
+  public: // AbstractColumnRenderer API
+    virtual void renderFirstLine( QPainter* painter, const PixelXRange& Xs, Line firstLineIndex );
+    virtual void renderNextLine( QPainter* painter );
 
   public:
-    void prepareRendering( const KPixelXs &Xs );
+    void prepareRendering( const PixelXRange& Xs );
 
   public:
-    //void renderLine( QPainter *painter, int lineIndex );
-    void renderLinePositions( QPainter *painter, int lineIndex, const KHE::Section &linePositions );
+    //void renderLine( QPainter* painter, int lineIndex );
+    void renderLinePositions( QPainter* painter, Line lineIndex, const LineRange& linePositions );
     /** paints a cursor based on the type of the byte.
       * @param byteIndex Index of the byte to paint the cursor for. If -1 a space is used as char.
       */
-    void renderCursor( QPainter *painter, int byteIndex );
+    void renderCursor( QPainter* painter, Address byteIndex );
     /** paints the byte with background.
       * @param byteIndex Index of the byte to paint. If -1 only the background is painted.
       */
-    void renderByte( QPainter *painter, int byteIndex );
+    void renderByte( QPainter* painter, Address byteIndex );
     /** paints the byte with background and a frame around.
       * @param byteIndex Index of the byte to paint the frame for. If -1 a space is used as char.
-      * @param Style the style of the framing
+      * @param style the style of the framing
       */
-    void renderFramedByte( QPainter *painter, int byteIndex, FrameStyle style );
+    void renderFramedByte( QPainter* painter, Address byteIndex, FrameStyle style );
 
 
   public: // modification access
@@ -100,129 +99,133 @@ class AbstractByteArrayColumnRenderer : public ColumnRenderer
       * @param groupSpacingWidth spacing between the groups in pixels
       * returns true if there was a change
       */
-    bool setSpacing( KPixelX byteSpacingWidth, int noOfGroupedBytes = 0, KPixelX groupSpacingWidth = 0 );
+    bool setSpacing( PixelX byteSpacingWidth, int noOfGroupedBytes = 0, PixelX groupSpacingWidth = 0 );
     /** sets the spacing between the bytes in the hex column
       * @param byteSpacingWidth spacing between the bytes in pixels
       * returns true if there was a change
       */
-    bool setByteSpacingWidth( KPixelX byteSpacingWidth );
+    bool setByteSpacingWidth( PixelX byteSpacingWidth );
     /** sets the number of grouped bytes in the hex column
-      * @param NewNoOfGroupedBytes numbers of grouped bytes, 0 means no grouping
+      * @param noOfGroupedBytes numbers of grouped bytes, 0 means no grouping
       * returns true if there was a change
       */
     bool setNoOfGroupedBytes( int noOfGroupedBytes );
     /** sets the spacing between the groups of bytes in the hex column
-      * @param GroupSpacingW spacing between the groups in pixels
+      * @param groupSpacingWidth spacing between the groups in pixels
       * returns true if there was a change
       */
-    bool setGroupSpacingWidth( KPixelX groupSpacingWidth );
+    bool setGroupSpacingWidth( PixelX groupSpacingWidth );
     /** sets width of digits and recalculates depend sizes
       * returns true if there was a change
       */
-    bool setDigitWidth( KPixelX digitWidth );
+    bool setDigitWidth( PixelX digitWidth );
     /** sets the metrics of the used font
       * @param digitWidth the new width of a digit
       * @param digitBaseLine the new baseline of the digits
       */
-    void setMetrics( KPixelX digitWidth, KPixelY digitBaseLine );
+    void setMetrics( PixelX digitWidth, PixelY digitBaseLine );
     /** */
-    void set( KHECore::AbstractByteArrayModel *byteArrayModel );
+    void set( AbstractByteArrayModel* byteArrayModel );
     /** creates new buffer for x-values; to be called on any change of NoOfBytesPerLine or metrics */
     void resetXBuffer();
     /** sets the codec to be used by the char column. */
-    void setCharCodec( const KHECore::CharCodec* charCodec );
+    void setCharCodec( const CharCodec* charCodec );
 
     void setByteTypeColored( bool byteTypeColored );
 
 
   public: // functional logic
     /** returns byte linePositions covered by pixels with absolute x-coord x */
-    KHE::Section linePositionsOfX( KPixelX x, KPixelX width ) const;
+    LinePositionRange linePositionsOfX( PixelX x, PixelX width ) const;
     /** returns byte pos at pixel with absolute x-coord x */
-    int linePositionOfX( KPixelX x ) const;
+    LinePosition linePositionOfX( PixelX x ) const;
     /** returns byte pos at pixel with absolute x-coord x, and sets the flag to true if we are closer to the right */
-    int magneticLinePositionOfX( KPixelX x ) const;
+    LinePosition magneticLinePositionOfX( PixelX x ) const;
     /** returns absolute x-coord of byte at position posInLine */
-    KPixelX xOfLinePosition( int posInLine ) const;
+    PixelX xOfLinePosition( LinePosition posInLine ) const;
     /** returns right absolute x-coord of byte at position posInLine */
-    KPixelX rightXOfLinePosition( int posInLine ) const;
+    PixelX rightXOfLinePosition( LinePosition posInLine ) const;
     /** returns byte pos at pixel with relative x-coord x */
-    int linePositionOfColumnX( KPixelX x ) const;
+    LinePosition linePositionOfColumnX( PixelX x ) const;
     /** returns byte linePositions covered by pixels with relative x-coord x */
-    KHE::Section linePositionsOfColumnXs( KPixelX x, KPixelX width ) const;
+    LinePositionRange linePositionsOfColumnXs( PixelX x, PixelX width ) const;
     /** returns relative x-coord of byte at position posInLine */
-    KPixelX columnXOfLinePosition( int posInLine ) const;
+    PixelX columnXOfLinePosition( LinePosition posInLine ) const;
     /** returns right relative x-coord of byte at position posInLine */
-    KPixelX columnRightXOfLinePosition( int posInLine ) const;
+    PixelX columnRightXOfLinePosition( LinePosition posInLine ) const;
     /** returns the linePositions that overlap with the x-coords relative to the view */
-    KHE::Section visibleLinePositions( KPixelX x, KPixelX width ) const;
+    LinePositionRange visibleLinePositions( PixelX x, PixelX width ) const;
     /** returns the */
-    KPixelXs xsOfLinePositionsInclSpaces( const KHE::Section &linePositions ) const;
+    PixelXRange xsOfLinePositionsInclSpaces( const LinePositionRange& linePositions ) const;
     /** */
-    KPixelXs columnXsOfLinePositionsInclSpaces( const KHE::Section &linePositions ) const;
+    PixelXRange columnXsOfLinePositionsInclSpaces( const LinePositionRange& linePositions ) const;
+
+    QRect byteRect( const Coord& coord ) const;
 
   public: // value access
-    KPixelX byteWidth() const;
-    KPixelX digitWidth() const;
-    KPixelX groupSpacingWidth() const;
-    KPixelX byteSpacingWidth() const;
+    PixelX byteWidth() const;
+    PixelX digitWidth() const;
+    PixelX groupSpacingWidth() const;
+    PixelX byteSpacingWidth() const;
     int noOfGroupedBytes() const;
 
-    int firstLinePos() const;
-    int lastLinePos()  const;
-    KHE::Section visibleLinePositions() const;
-    const ByteArrayTableLayout *layout() const;
+    LinePosition firstLinePos() const;
+    LinePosition lastLinePos()  const;
+    LinePositionRange visibleLinePositions() const;
+    const ByteArrayTableLayout* layout() const;
     bool isByteTypeColored() const;
 
 
   protected: // API to be redefined
-    virtual void renderByteText( QPainter *painter, char byte, KHECore::KChar charByte, const QColor &color ) const = 0;
+    virtual void renderByteText( QPainter* painter, Byte byte, Character charByte, const QColor& color ) const = 0;
     /** default implementation sets byte width to one digit width */
     virtual void recalcByteWidth();
 
 
   protected:
-    void renderPlain( QPainter *painter, const KHE::Section &linePositions, int byteIndex );
-    void renderSelection( QPainter *painter, const KHE::Section &linePositions, int byteIndex, int flag );
-    void renderMarking( QPainter *painter, const KHE::Section &linePositions, int byteIndex, int flag );
-    void renderRange( QPainter *painter, const QBrush &brush, const KHE::Section &linePositions, int flag );
-    void renderBookmark( QPainter *painter, const QBrush &brush );
+    void renderPlain( QPainter* painter, const LinePositionRange& linePositions, Address byteIndex );
+    void renderSelection( QPainter* painter, const LinePositionRange& linePositions, Address byteIndex, int flag );
+    void renderMarking( QPainter* painter, const LinePositionRange& linePositions, Address byteIndex, int flag );
+    void renderRange( QPainter* painter, const QBrush& brush, const LinePositionRange& linePositions, int flag );
+    void renderSelectionSpaceBehind( QPainter* painter, LinePosition linePosition );
+    void renderSpaceBehind( QPainter* painter, const QBrush& brush, LinePosition linePosition );
+    void renderBookmark( QPainter* painter, const QBrush& brush );
 
     void recalcX();
 
-    bool isSelected( const KHE::Section &range, KHE::Section *selection, unsigned int *flag ) const;
-    bool isMarked( const KHE::Section &range, KHE::Section *marking, unsigned int *flag ) const;
+    bool getNextSelectedAddressRange( AddressRange* selectedRange, unsigned int* flag, const AddressRange& range ) const;
+    bool getNextMarkedAddressRange( AddressRange* markedRange, unsigned int* flag, const AddressRange& range ) const;
 
     void setByteWidth( int byteWidth );
 
 
   protected:
     /** pointer to the buffer */
-    KHECore::AbstractByteArrayModel *mByteArrayModel;
+    AbstractByteArrayModel* mByteArrayModel;
     /** pointer to the layout */
-    const ByteArrayTableLayout *mLayout;
+    const ByteArrayTableLayout* mLayout;
     /** pointer to the ranges */
     ByteArrayTableRanges* mRanges;
     /** */
-    KHECore::Bookmarkable *mBookmarks;
+    Bookmarkable* mBookmarks;
     /** */
-    const KHECore::CharCodec* mCharCodec;
+    const CharCodec* mCharCodec;
 
     /** */
-    KPixelX mDigitWidth;
+    PixelX mDigitWidth;
     /** */
-    KPixelY mDigitBaseLine;
+    PixelY mDigitBaseLine;
 
 
   protected:  // individual data
     /** total width of byte display in pixel */
-    KPixelX mByteWidth;
+    PixelX mByteWidth;
     /** width of inserting cursor in pixel */
-//     KPixelX mCursorWidth;
+//     PixelX mCursorWidth;
     /** size of the line margin */
-    KPixelX mByteSpacingWidth;
+    PixelX mByteSpacingWidth;
     /** width of spacing in pixel */
-    KPixelX mGroupSpacingWidth;
+    PixelX mGroupSpacingWidth;
 
     /** number of grouped bytes */
     int mNoOfGroupedBytes;
@@ -230,38 +233,38 @@ class AbstractByteArrayColumnRenderer : public ColumnRenderer
     /** pointer to array with buffered linePositions (relative to column position)
       * a spacing gets assigned to the left byte -> ...c|c|c |c|c...
       */
-    KPixelX *mLinePosLeftPixelX;
-    KPixelX *mLinePosRightPixelX;
+    PixelX* mLinePosLeftPixelX;
+    PixelX* mLinePosRightPixelX;
     /** index of right position */
-    int mLastLinePos;
+    LinePosition mLastLinePos;
 
     /** */
     bool mByteTypeColored;
 
 
   protected: // buffering rendering data
-    KHE::Section mRenderLinePositions;
-    int mRenderLine;
-    KPixelX mRenderX;
-    KPixelX mRenderWidth;
+    LinePositionRange mRenderLinePositions;
+    Line mRenderLine;
+    PixelX mRenderX;
+    PixelX mRenderWidth;
     int mSpacingTrigger;
 };
 
 
-inline KPixelX AbstractByteArrayColumnRenderer::byteWidth()         const { return mByteWidth; }
-inline KPixelX AbstractByteArrayColumnRenderer::digitWidth()        const { return mDigitWidth; }
-inline KPixelX AbstractByteArrayColumnRenderer::byteSpacingWidth()  const { return mByteSpacingWidth; }
-inline KPixelX AbstractByteArrayColumnRenderer::groupSpacingWidth() const { return mGroupSpacingWidth; }
+inline PixelX AbstractByteArrayColumnRenderer::byteWidth()         const { return mByteWidth; }
+inline PixelX AbstractByteArrayColumnRenderer::digitWidth()        const { return mDigitWidth; }
+inline PixelX AbstractByteArrayColumnRenderer::byteSpacingWidth()  const { return mByteSpacingWidth; }
+inline PixelX AbstractByteArrayColumnRenderer::groupSpacingWidth() const { return mGroupSpacingWidth; }
 
 inline int AbstractByteArrayColumnRenderer::noOfGroupedBytes()      const { return mNoOfGroupedBytes; }
 
-inline int AbstractByteArrayColumnRenderer::firstLinePos() const { return mRenderLinePositions.start(); }
-inline int AbstractByteArrayColumnRenderer::lastLinePos()  const { return mRenderLinePositions.end(); }
-inline KHE::Section AbstractByteArrayColumnRenderer::visibleLinePositions() const { return mRenderLinePositions; }
+inline LinePosition AbstractByteArrayColumnRenderer::firstLinePos() const { return mRenderLinePositions.start(); }
+inline LinePosition AbstractByteArrayColumnRenderer::lastLinePos()  const { return mRenderLinePositions.end(); }
+inline LinePositionRange AbstractByteArrayColumnRenderer::visibleLinePositions() const { return mRenderLinePositions; }
 
-inline const ByteArrayTableLayout *AbstractByteArrayColumnRenderer::layout() const { return mLayout; }
+inline const ByteArrayTableLayout* AbstractByteArrayColumnRenderer::layout() const { return mLayout; }
 
-inline void AbstractByteArrayColumnRenderer::setCharCodec( const KHECore::CharCodec* charCodec )
+inline void AbstractByteArrayColumnRenderer::setCharCodec( const CharCodec* charCodec )
 {
     mCharCodec = charCodec;
 }
