@@ -1,7 +1,7 @@
 /*
     This file is part of the Okteta Gui library, part of the KDE project.
 
-    Copyright 2008 Friedrich W. H. Kossebau <kossebau@kde.org>
+    Copyright 2008-2009 Friedrich W. H. Kossebau <kossebau@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -20,28 +20,27 @@
     License along with this library. If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef KHE_UI_ABSTRACTBYTEARRAYVIEW_H
-#define KHE_UI_ABSTRACTBYTEARRAYVIEW_H
+#ifndef OKTETA_ABSTRACTBYTEARRAYVIEW_H
+#define OKTETA_ABSTRACTBYTEARRAYVIEW_H
 
 // lib
 #include "columnsview.h"
-
-namespace KHECore {
-class Bookmark;
-class ValueCodec;
-class CharCodec;
-class AbstractByteArrayModel;
-}
-namespace KHE {
-class ArrayChangeMetricsList;
-class Section;
-}
+// Okteta core
+#include "addressrange.h"
+// Qt
+#include <QtGui/QClipboard>
 
 class QMimeData;
 class QByteArray;
 
-namespace KHEUI
+namespace Okteta
 {
+class Bookmark;
+class ValueCodec;
+class CharCodec;
+class AbstractByteArrayModel;
+class ArrayChangeMetricsList;
+
 class KController;
 class AbstractWheelController;
 class ByteArrayTableLayout;
@@ -61,6 +60,9 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
   friend class KValueEditor;
   friend class KCharEditor;
   friend class Dropper;
+  friend class MouseNavigator;
+  friend class MousePaster;
+  friend class Dragger;
 
   Q_OBJECT
   Q_PROPERTY( bool OverwriteMode READ isOverwriteMode WRITE setOverwriteMode )
@@ -71,26 +73,39 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
   //Q_PROPERTY( bool hasSelectedData READ hasSelectedData )
   //Q_PROPERTY( QByteArray SelectedData READ selectedData )
   Q_PROPERTY( bool TabChangesFocus READ tabChangesFocus WRITE setTabChangesFocus )
-  Q_PROPERTY( ResizeStyle ResizeStyle READ resizeStyle WRITE setResizeStyle )
+  Q_PROPERTY( LayoutStyle LayoutStyle READ layoutStyle WRITE setLayoutStyle )
   Q_PROPERTY( int NoOfBytesPerLine READ noOfBytesPerLine WRITE setNoOfBytesPerLine )
   Q_PROPERTY( int StartOffset READ startOffset WRITE setStartOffset )
   Q_PROPERTY( int FirstLineOffset READ firstLineOffset WRITE setFirstLineOffset )
-  Q_PROPERTY( int VisibleCodings READ visibleCodings WRITE setVisibleCodings )
-  Q_ENUMS( ValueCoding CharCoding ResizeStyle )
+  Q_PROPERTY( bool OffsetColumnVisible READ offsetColumnVisible WRITE toggleOffsetColumn )
+  Q_PROPERTY( CodingTypes VisibleCodings READ visibleCodings WRITE setVisibleCodings )
+  Q_ENUMS( ValueCoding CharCoding LayoutStyle CodingTypes )
+
+  Q_PROPERTY( bool ByteTypeColored READ isByteTypeColored WRITE setByteTypeColored )
+  // value column
+  Q_PROPERTY( ValueCoding Coding READ valueCoding WRITE setValueCoding )
+  Q_PROPERTY( int ByteSpacingWidth READ byteSpacingWidth WRITE setByteSpacingWidth )
+  Q_PROPERTY( int NoOfGroupedBytes READ noOfGroupedBytes WRITE setNoOfGroupedBytes )
+  Q_PROPERTY( int GroupSpacingWidth READ groupSpacingWidth WRITE setGroupSpacingWidth )
+  Q_PROPERTY( int BinaryGapWidth READ binaryGapWidth WRITE setBinaryGapWidth )
+  // char column
+  Q_PROPERTY( bool ShowNonprinting READ showsNonprinting WRITE setShowsNonprinting )
+  Q_PROPERTY( QChar SubstituteChar READ substituteChar WRITE setSubstituteChar )
 
   public:
     enum ValueCoding { HexadecimalCoding=0, DecimalCoding=1, OctalCoding=2, BinaryCoding=3, MaxCodingId=0xFFFF };
     enum CharCoding { LocalEncoding=0, ISO8859_1Encoding=1, EBCDIC1047Encoding=2,
                      StartOfOwnEncoding=0x8000, MaxEncodingId=0xFFFF };
-    enum ResizeStyle { NoResize=0, LockGrouping=1, FullSizeUsage=2, MaxResizeStyleId=0xFF };
+    enum LayoutStyle { FixedLayoutStyle=0, WrapOnlyByteGroupsLayoutStyle=1, FullSizeLayoutStyle=2, LastUserLayout=0xFF };
 
-    enum CodingTypeId { NoCodingId=0, ValueCodingId=1, CharCodingId=2, BothCodingsId=ValueCodingId|CharCodingId };
+    enum CodingTypeId { NoCodingId=0, ValueCodingId=1, CharCodingId=2 };
+    enum CodingTypes { OnlyValueCoding = ValueCodingId, OnlyCharCoding = CharCodingId, ValueAndCharCodings=ValueCodingId|CharCodingId };
 
   public:
     virtual ~AbstractByteArrayView();
 
   public: // value access
-    KHECore::AbstractByteArrayModel* byteArrayModel() const;
+    Okteta::AbstractByteArrayModel* byteArrayModel() const;
     bool isModified() const;
 
     bool isOverwriteMode() const;
@@ -98,24 +113,28 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     bool isReadOnly() const;
 
     /** returns the index of the cursor position */
-    int cursorPosition() const;
+    Address cursorPosition() const;
     /***/
     bool isCursorBehind() const;
 
-    int startOffset() const;
-    int firstLineOffset() const;
+    Address startOffset() const;
+    Address firstLineOffset() const;
     int noOfBytesPerLine() const;
 
-    ResizeStyle resizeStyle() const;
+    LayoutStyle layoutStyle() const;
 
     bool tabChangesFocus() const;
 
-    int visibleCodings() const;
+    CodingTypes visibleCodings() const;
     CodingTypeId activeCoding() const;
 
+    bool offsetColumnVisible() const;
+
     /** returns true if there is a selected range in the array */
     bool hasSelectedData() const;
-    KHE::Section selection() const;
+    AddressRange selection() const;
+
+    AddressRange marking() const;
 
     ValueCoding valueCoding() const;
     /**
@@ -130,13 +149,11 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     double zoomLevel() const;
 
   public: // value access API
-    virtual bool offsetColumnVisible() const = 0;
-
   // value column
-    virtual int/*KPixelX*/ byteSpacingWidth() const = 0;
+    virtual int/*PixelX*/ byteSpacingWidth() const = 0;
     virtual int noOfGroupedBytes() const = 0;
-    virtual int/*KPixelX*/ groupSpacingWidth() const = 0;
-    virtual int/*KPixelX*/ binaryGapWidth() const = 0;
+    virtual int/*PixelX*/ groupSpacingWidth() const = 0;
+    virtual int/*PixelX*/ binaryGapWidth() const = 0;
 
   // char column
     /** reports if "non-printing" chars are displayed in the char column
@@ -155,47 +172,44 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
 
 
   public:
-    virtual void setByteArrayModel( KHECore::AbstractByteArrayModel* byteArrayModel );
-
-    /** switches the Offset column on/off */
-    virtual void toggleOffsetColumn( bool offsetColumnVisible ) = 0;
+    virtual void setByteArrayModel( Okteta::AbstractByteArrayModel* byteArrayModel );
 
   // setting parameters
   // value column parameters
     /** sets the spacing between the bytes in the value column
-      * @param BSW spacing between the bytes in pixels
+      * @param byteSpacingWidth spacing between the bytes in pixels
       * default is 3
       */
-    virtual void setByteSpacingWidth( int/*KPixelX*/ byteSpacingWidth ) = 0;
+    virtual void setByteSpacingWidth( int/*PixelX*/ byteSpacingWidth ) = 0;
     /** sets the number of grouped bytes in the value column
-      * @param NoGB numbers of grouped bytes, 0 means no grouping
+      * @param noOfGroupedBytes numbers of grouped bytes, 0 means no grouping
       * default is 4
       */
     virtual void setNoOfGroupedBytes( int noOfGroupedBytes ) = 0;
     /** sets the spacing between the groups of bytes in the value column
-      * @param GSW spacing between the groups in pixels
+      * @param groupSpacingWidth spacing between the groups in pixels
       * default is 9
       */
-    virtual void setGroupSpacingWidth( int/*KPixelX*/ groupSpacingWidth ) = 0;
+    virtual void setGroupSpacingWidth( int/*PixelX*/ groupSpacingWidth ) = 0;
     /** sets the spacing in the middle of a binary byte in the value column
-      * @param BinaryGapW spacing in the middle of a binary in pixels
+      * @param binaryGapWidth spacing in the middle of a binary in pixels
       * returns true if there was a change
       */
     virtual void setBinaryGapWidth( int binaryGapWidth ) = 0;
     /** sets the spacing in the value column
-      * @param ByteSpacingWidth spacing between the bytes in pixels
-      * @param NoOfGroupedBytes numbers of grouped bytes, 0 means no grouping
-      * @param GroupSpacingWidth spacing between the groups in pixels
+      * @param byteSpacingWidth spacing between the bytes in pixels
+      * @param noOfGroupedBytes numbers of grouped bytes, 0 means no grouping
+      * @param groupSpacingWidth spacing between the groups in pixels
       * Default is 4 for NoOfGroupedBytes
       */
-    virtual void setBufferSpacing( int/*KPixelX*/ byteSpacingWidth, int noOfGroupedBytes = 0, int/*KPixelX*/ groupSpacingWidth = 0 ) = 0;
-    /** sets the format of the value column. Default is KHE::HexadecimalCoding */
+    virtual void setBufferSpacing( int/*PixelX*/ byteSpacingWidth, int noOfGroupedBytes = 0, int/*PixelX*/ groupSpacingWidth = 0 ) = 0;
+    /** sets the format of the value column. Default is Okteta::HexadecimalCoding */
     virtual void setValueCoding( ValueCoding valueCoding ) = 0;
   // char column parameters
     /** sets whether control chars or "non-printing" chars should be displayed in the char column
       * with their corresponding character. Currently this simply means all chars with value <32,
       * as known from the ASCII.
-      * @param SU
+      * @param showsNonprinting
       * returns true if there was a change
       */
     virtual void setShowsNonprinting( bool showsNonprinting = true ) = 0;
@@ -207,12 +221,12 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
      * returns true if there was a change
      */
     virtual void setUndefinedChar( QChar undefinedChar ) = 0;
-    /** sets the encoding of the char column. Default is KHE::LocalEncoding.
+    /** sets the encoding of the char column. Default is Okteta::LocalEncoding.
       * If the encoding is not available the format will not be changed. */
     virtual void setCharCoding( CharCoding charCoding ) = 0;
-    /** sets the encoding of the char column. Default is KHE::LocalEncoding.
+    /** sets the encoding of the char column. Default is Okteta::LocalEncoding.
       * If the encoding is not available the format will not be changed.
-      * @param Encoding name of the encoding
+      * @param charCodingName name of the encoding
       */
     virtual void setCharCoding( const QString& charCodingName ) = 0;
     virtual void setByteTypeColored( bool isColored ) = 0;
@@ -221,8 +235,8 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     /** sets whether the data should be treated modified or not */
     void setModified( bool modified );
 
-    /** sets the resizestyle for the value column. Default is KHE::FullSizeUsage */
-    void setResizeStyle( ResizeStyle resizeStyle );
+    /** sets the resizestyle for the value column. Default is Okteta::FullSizeLayoutStyle */
+    void setLayoutStyle( LayoutStyle layoutStyle );
     /** sets whether the widget is readonly or not, Default is true.
       * If the databuffer which is worked on can't be written the widget stays readonly
       */
@@ -232,12 +246,12 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     /** sets whether the widget is in overwrite mode or not. Default is true. */
     void setOverwriteMode( bool overwriteMode );
 
-    /** sets the number of bytes per line, switching the resize style to KHE::NoResize */
+    /** sets the number of bytes per line, switching the resize style to Okteta::FixedLayoutStyle */
     void setNoOfBytesPerLine( int noOfBytesPerLine );
     /** sets absolut offset of the data */
-    void setStartOffset( int startOffset );
+    void setStartOffset( Address startOffset );
     /** sets offset of the char in the upper left corner */
-    void setFirstLineOffset( int firstLineOffset );
+    void setFirstLineOffset( Address firstLineOffset );
 
     /** sets whether on a tab key there should be switched from the char column back to the value column
       * or be switched to the next focusable widget. Default is false
@@ -249,6 +263,9 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     /** */
     void setVisibleCodings( int visibleCodings );
 
+    /** switches the Offset column on/off */
+    void toggleOffsetColumn( bool offsetColumnVisible );
+
   public:
     /** 
       * @return deep copy of the selected data 
@@ -267,15 +284,24 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
       * @param index 
       * @param isBehind 
       */
-    void setCursorPosition( int index, bool isBehind = false );
-    void setSelectionCursorPosition( int index );
+    void setCursorPosition( Address index, bool isBehind = false );
+    void setSelectionCursorPosition( Address index );
 
     /** de-/selects all data */
     void selectAll( bool select );
 
-    void setSelection( int start, int end );
+    void setSelection( Address start, Address end );
+    void setSelection( const AddressRange& selection );
     /** selects word at index, returns true if there is one */
-    bool selectWord( /*unsigned*/ int index /*, Chartype*/ );
+    bool selectWord( /*unsigned*/ Address index /*, Chartype*/ );
+
+    void setMarking( Address start, Address end );
+    void setMarking( const AddressRange& marking );
+
+    /**
+     * @param ensureStartVisible if true , otherwise the end
+     */
+    void ensureVisible( const AddressRange& range, bool ensureStartVisible = false );
     /** scrolls the view as much as needed to have the cursor fully visible */
     void ensureCursorVisible();
     /** puts the cursor in the column at the pos of Point (in absolute coord), does not handle the drawing */
@@ -315,21 +341,27 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
   public: // QWidget API
     virtual QSize sizeHint() const;
 
+  public: // logic value service
+    /** detects the index of the byte at the given point
+      * @param point in viewport coordinate system
+      * @return index of the byte that covers the point
+      */
+    Address indexByPoint( const QPoint& point ) const;
 
   Q_SIGNALS:
     /** Index of the byte that was clicked */
-    void clicked( int index );
+    void clicked( Okteta::Address index );
     /** Index of the byte that was double clicked */
-    void doubleClicked( int index );
+    void doubleClicked( Okteta::Address index );
 
-    void cursorPositionChanged( int index );
+    void cursorPositionChanged( Okteta::Address index );
     /** */
     void overwriteModeChanged( bool newOverwriteMode );
     /** */
     void readOnlyChanged( bool isReadOnly );
     /** selection has changed */
-    void selectionChanged( bool hasSelection );
-    void selectionChanged( const KHE::Section& selection );
+    void hasSelectedDataChanged( bool hasSelectedData );
+    void selectionChanged( const Okteta::AddressRange& selection );
     /** there is a cut available or not */
     void cutAvailable( bool Really );
     /** there is a copy available or not */
@@ -338,21 +370,28 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     void charCodecChanged( const QString& codecName );
     /** */
     void valueCodingChanged( int valueCoding );
+    void focusChanged( bool hasFocus );
 
   protected:
     void finishByteEdit();
     void emitSelectionSignals();
     void updateChanged();
+    void copyToClipboard( QClipboard::Mode mode ) const;
+    void pasteFromClipboard( QClipboard::Mode mode );
 
   protected:
-    const KHECore::ValueCodec* valueCodec() const;
-    const KHECore::CharCodec* charCodec() const;
+    const Okteta::ValueCodec* valueCodec() const;
+    const Okteta::CharCodec* charCodec() const;
     ByteArrayTableCursor* tableCursor() const;
     ByteArrayTableRanges* tableRanges() const;
     ByteArrayTableLayout* layout() const;
 
   protected: // QWidget API
     virtual void keyPressEvent( QKeyEvent* keyEvent );
+    virtual void mousePressEvent( QMouseEvent* mousePressEvent );
+    virtual void mouseReleaseEvent( QMouseEvent* mouseReleaseEvent );
+    virtual void mouseMoveEvent( QMouseEvent* mouseMoveEvent );
+    virtual void mouseDoubleClickEvent( QMouseEvent* mouseDoubleClickEvent );
     /// reimplemented to catch Tab and BackTab keys, which otherwise gets stolen
     virtual bool event( QEvent* event );
     virtual void showEvent( QShowEvent* showEvent );
@@ -363,6 +402,7 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
     virtual void dragMoveEvent( QDragMoveEvent* dragMoveEvent) ;
     virtual void dragLeaveEvent( QDragLeaveEvent* dragLeaveEvent );
     virtual void dropEvent( QDropEvent* dropEvent );
+//    virtual void contextMenuEvent( QContextMenuEvent* contextMenuEvent );
 
   protected: // QAbstractScrollArea API
     virtual void wheelEvent( QWheelEvent *e );
@@ -380,10 +420,12 @@ class OKTETAGUI_EXPORT AbstractByteArrayView : public ColumnsView
   private:
     Q_DECLARE_PRIVATE( AbstractByteArrayView )
     Q_PRIVATE_SLOT( d_func(), void adaptController() )
+    Q_PRIVATE_SLOT( d_func(), void blinkCursor() )
     Q_PRIVATE_SLOT( d_func(), void onByteArrayReadOnlyChange(bool isByteArrayReadOnly) )
-    Q_PRIVATE_SLOT( d_func(), void onContentsChanged( const KHE::ArrayChangeMetricsList &changeList ) )
-    Q_PRIVATE_SLOT( d_func(), void onBookmarksChange( const QList<KHECore::Bookmark> &bookmarks ) )
+    Q_PRIVATE_SLOT( d_func(), void onContentsChanged( const Okteta::ArrayChangeMetricsList& changeList ) )
+    Q_PRIVATE_SLOT( d_func(), void onBookmarksChange( const QList<Okteta::Bookmark>& bookmarks ) )
     Q_PRIVATE_SLOT( d_func(), void onRevertedToVersionIndex( int versionIndex ) )
+//     Q_PRIVATE_SLOT( d_func(), void onClipboardChanged() )
 };
 
 }
