@@ -1,7 +1,7 @@
 /*
     This file is part of the Okteta Gui library, part of the KDE project.
 
-    Copyright 2008 Friedrich W. H. Kossebau <kossebau@kde.org>
+    Copyright 2008-2009 Friedrich W. H. Kossebau <kossebau@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -25,44 +25,25 @@
 // lib
 #include "bordercolumnrenderer.h"
 #include "widgetcolumnstylist.h"
-#include "bytearraytableranges.h"
-#include "bytearraytablelayout.h"
-#include "controller/kvalueeditor.h"
 #include "controller/dropper.h"
 #include "kcursor.h"
 // Okteta core
-#include <abstractbytearraymodel.h>
 #include <valuecodec.h>
-#include <charcodec.h>
-#include <wordbytearrayservice.h>
 // KDE
 #include <KGlobalSettings>
 // Qt
-#include <QtGui/QApplication>
 #include <QtGui/QStyle>
 #include <QtGui/QPainter>
-#include <QtGui/QCursor>
-#include <QtGui/QClipboard>
 #include <QtGui/QScrollBar>
-#include <QtGui/QMouseEvent>
-#include <QtCore/QListIterator>
-#include <QtCore/QTimer>
 
 
 namespace Okteta
 {
-static const int DefaultScrollTimerPeriod = 100;
 static const int InsertCursorWidth = 2;
 
 
 ByteArrayRowViewPrivate::ByteArrayRowViewPrivate( ByteArrayRowView* parent )
-: AbstractByteArrayViewPrivate( parent ),
-   mCursorPixmaps( new KCursor() ),
-   mMousePressed( false ),
-   mInDoubleClick( false ),
-   mInDnD( false ),
-   mDragStartPossible( false ),
-   mBlinkCursorVisible( false )
+  : AbstractByteArrayViewPrivate( parent )
 {
 }
 
@@ -70,23 +51,12 @@ void ByteArrayRowViewPrivate::init()
 {
    Q_Q( ByteArrayRowView );
 
-    mCursorBlinkTimer = new QTimer( q );
-    mScrollTimer = new QTimer( q );
-    mDragStartTimer = new QTimer( q );
-    mTrippleClickTimer = new QTimer( q );
-
-    mStylist = new WidgetColumnStylist( q );
-
     // creating the columns in the needed order
-    mOffsetColumn =
-        new OffsetColumnRenderer( mStylist, mTableLayout, OffsetFormat::Hexadecimal );
-    mBorderColumn =
-        new BorderColumnRenderer( mStylist, false );
     mByteArrayColumn =
         new ByteArrayRowColumnRenderer( mStylist, mByteArrayModel, mTableLayout, mTableRanges );
 
     q->addColumn( mOffsetColumn );
-    q->addColumn( mBorderColumn );
+    q->addColumn( mOffsetBorderColumn );
     q->addColumn( mByteArrayColumn );
 
     // select the active column
@@ -100,15 +70,8 @@ void ByteArrayRowViewPrivate::init()
 
     adaptController();
 
+    // do here not in base class, as handleFontChange needs this init run before
     q->setFont( KGlobalSettings::fixedFont() );
-
-    q->connect( mCursorBlinkTimer, SIGNAL(timeout()), q, SLOT(blinkCursor()) );
-    q->connect( mScrollTimer,      SIGNAL(timeout()), q, SLOT(autoScrollTimerDone()) );
-    q->connect( mDragStartTimer,   SIGNAL(timeout()), q, SLOT(startDrag()) );
-    mDragStartTimer->setSingleShot( true );
-    mTrippleClickTimer->setSingleShot( true );
-
-    q->setAcceptDrops( true );
 }
 
 AbstractByteArrayView::CodingTypes ByteArrayRowViewPrivate::visibleCodings() const { return mByteArrayColumn->visibleCodings(); }
@@ -317,19 +280,6 @@ void ByteArrayRowViewPrivate::adjustToLayoutNoOfBytesPerLine()
 }
 
 
-void ByteArrayRowViewPrivate::toggleOffsetColumn( bool showOffsetColumn )
-{
-    const bool isVisible = mOffsetColumn->isVisible();
-    // no change?
-    if( isVisible == showOffsetColumn )
-        return;
-
-    mOffsetColumn->setVisible( showOffsetColumn );
-
-    updateViewByWidth();
-}
-
-
 QSize ByteArrayRowViewPrivate::minimumSizeHint() const
 {
     Q_Q( const ByteArrayRowView );
@@ -337,7 +287,7 @@ QSize ByteArrayRowViewPrivate::minimumSizeHint() const
     // TODO: better minimal width (visibility!)
     const int minWidth =
         mOffsetColumn->visibleWidth()
-        + mBorderColumn->visibleWidth()
+        + mOffsetBorderColumn->visibleWidth()
         + mByteArrayColumn->byteWidth();
     const int minHeight =
         q->lineHeight()
@@ -354,7 +304,7 @@ int ByteArrayRowViewPrivate::fittingBytesPerLine() const
     const QSize newSize = q->maximumViewportSize();
     const PixelX reservedWidth =
         mOffsetColumn->visibleWidth()
-        + mBorderColumn->visibleWidth();
+        + mOffsetBorderColumn->visibleWidth();
 
     // abstract offset and border columns width
     const PixelX fullWidth = newSize.width() - reservedWidth;
@@ -384,9 +334,9 @@ int ByteArrayRowViewPrivate::fittingBytesPerLine() const
     // no grouping?
     if( noOfGroupedBytes == 0 )
     {
-        // faking grouping by 1
+        // fake no grouping by grouping with 1 and using byteSpacingWidth
         noOfGroupedBytes = 1;
-        groupSpacingWidth = 0;
+        groupSpacingWidth = byteSpacingWidth;
     }
     else
         groupSpacingWidth = mByteArrayColumn->groupSpacingWidth();
@@ -593,25 +543,6 @@ void ByteArrayRowViewPrivate::blinkCursor()
 }
 
 
-void ByteArrayRowViewPrivate::startCursor()
-{
-    mCursorPaused = false;
-
-    updateCursors();
-
-    mCursorBlinkTimer->start( QApplication::cursorFlashTime()/2 );
-}
-
-
-void ByteArrayRowViewPrivate::unpauseCursor()
-{
-    mCursorPaused = false;
-
-    if( mCursorBlinkTimer->isActive() )
-        updateCursors();
-}
-
-
 void ByteArrayRowViewPrivate::updateCursors()
 {
     createCursorPixmaps();
@@ -622,14 +553,6 @@ void ByteArrayRowViewPrivate::updateCursors()
 }
 
 
-void ByteArrayRowViewPrivate::stopCursor()
-{
-    mCursorBlinkTimer->stop();
-
-    pauseCursor();
-}
-
-
 void ByteArrayRowViewPrivate::pauseCursor()
 {
     mCursorPaused = true;
@@ -905,387 +828,8 @@ void ByteArrayRowViewPrivate::ensureVisible( const ByteArrayRowColumnRenderer& c
 }
 
 
-#if 0
-void ByteArrayRowViewPrivate::clipboardChanged()
-{
-    Q_Q( ByteArrayRowView );
-
-    // don't listen to selection changes
-    q->disconnect( QApplication::clipboard(), SIGNAL(selectionChanged()) );
-    selectAll( false );
-}
-#endif
-
-void ByteArrayRowViewPrivate::mousePressEvent( QMouseEvent* mouseEvent )
-{
-    Q_Q( ByteArrayRowView );
-
-    const bool oldHasSelection = mTableRanges->hasSelection();
-
-    pauseCursor();
-    mValueEditor->finishEdit();
-
-    // care about a left button press?
-    if( mouseEvent->button() == Qt::LeftButton )
-    {
-        mMousePressed = true;
-
-        // select whole line?
-        if( mTrippleClickTimer->isActive()
-            && (mouseEvent->globalPos()-mDoubleClickPoint).manhattanLength() < QApplication::startDragDistance() )
-        {
-            mTrippleClickTimer->stop();
-            const Address indexAtFirstDoubleClickLinePosition = mTableLayout->indexAtFirstLinePosition( mDoubleClickLine );
-            mTableRanges->setSelectionStart( indexAtFirstDoubleClickLinePosition );
-            mTableCursor->gotoIndex( indexAtFirstDoubleClickLinePosition );
-            mTableCursor->gotoLineEnd();
-            mTableRanges->setSelectionEnd( cursorPosition() );
-            updateChanged();
-
-            unpauseCursor();
-
-            const bool newHasSelection = mTableRanges->hasSelection();
-            emit q->cursorPositionChanged( cursorPosition() );
-            emit q->selectionChanged( mTableRanges->selection() );
-            if( oldHasSelection != newHasSelection )
-            {
-                if( !mOverWrite ) emit q->cutAvailable( newHasSelection );
-                emit q->copyAvailable( newHasSelection );
-                emit q->hasSelectedDataChanged( newHasSelection );
-            }
-            return;
-        }
-
-        const QPoint mousePoint = q->viewportToColumns( mouseEvent->pos() );
-
-        // start of a drag perhaps?
-        if( mTableRanges->hasSelection() && mTableRanges->selectionIncludes(indexByPoint( mousePoint )) )
-        {
-            mDragStartPossible = true;
-            mDragStartTimer->start( QApplication::startDragTime() );
-            mDragStartPoint = mousePoint;
-        }
-        else
-        {
-            placeCursor( mousePoint );
-            ensureCursorVisible();
-
-            const Address realIndex = mTableCursor->realIndex();
-            if( mTableRanges->selectionStarted() )
-            {
-                if( mouseEvent->modifiers() & Qt::SHIFT )
-                    mTableRanges->setSelectionEnd( realIndex );
-                else
-                {
-                    mTableRanges->removeSelection();
-                    mTableRanges->setSelectionStart( realIndex );
-                }
-            }
-            else // start of a new selection possible
-            {
-                mTableRanges->setSelectionStart( realIndex );
-
-                if( !isEffectiveReadOnly() && (mouseEvent->modifiers()&Qt::SHIFT) ) // TODO: why only for readwrite?
-                    mTableRanges->setSelectionEnd( realIndex );
-            }
-
-            mTableRanges->removeFurtherSelections();
-        }
-    }
-    else if( mouseEvent->button() == Qt::MidButton )
-        mTableRanges->removeSelection();
-
-    if( mTableRanges->isModified() )
-    {
-        updateChanged();
-        q->viewport()->setCursor( isEffectiveReadOnly() ? Qt::ArrowCursor : Qt::IBeamCursor );
-    }
-
-    unpauseCursor();
-
-    const bool newHasSelection = mTableRanges->hasSelection();
-    emit q->selectionChanged( mTableRanges->selection() );
-    if( oldHasSelection != newHasSelection )
-    {
-        if( !mOverWrite ) emit q->cutAvailable( newHasSelection );
-        emit q->copyAvailable( newHasSelection );
-        emit q->hasSelectedDataChanged( newHasSelection );
-    }
-}
-
-
-void ByteArrayRowViewPrivate::mouseMoveEvent( QMouseEvent *mouseEvent )
-{
-    Q_Q( ByteArrayRowView );
-
-    const QPoint movePoint = q->viewportToColumns( mouseEvent->pos() );
-
-    if( mMousePressed )
-    {
-        if( mDragStartPossible )
-        {
-            mDragStartTimer->stop();
-            // moved enough for a drag?
-            if( (movePoint-mDragStartPoint).manhattanLength() > QApplication::startDragDistance() )
-                startDrag();
-            if( !isEffectiveReadOnly() )
-                q->viewport()->setCursor( Qt::IBeamCursor );
-            return;
-        }
-        // selecting
-        handleMouseMove( movePoint );
-    }
-    else if( !isEffectiveReadOnly() )
-    {
-        // visual feedback for possible dragging
-        const bool InSelection =
-            mTableRanges->hasSelection() && mTableRanges->selectionIncludes( indexByPoint(movePoint) );
-        q->viewport()->setCursor( InSelection?Qt::ArrowCursor:Qt::IBeamCursor );
-    }
-}
-
-
-void ByteArrayRowViewPrivate::mouseReleaseEvent( QMouseEvent* mouseEvent )
-{
-    Q_Q( ByteArrayRowView );
-
-    const bool oldHasSelection = mTableRanges->hasSelection();
-    const QPoint releasePoint = q->viewportToColumns( mouseEvent->pos() );
-
-    // this is not the release of a doubleclick so we need to process it?
-    if( !mInDoubleClick )
-    {
-        const int line = q->lineAt( releasePoint.y() );
-        const int pos = mByteArrayColumn->linePositionOfX( releasePoint.x() ); // TODO: can we be sure here about the active column?
-        const Address index = mTableLayout->indexAtCCoord( Coord(pos,line) ); // TODO: can this be another index than the one of the cursor???
-        emit q->clicked( index );
-    }
-
-    if( mMousePressed )
-    {
-        mMousePressed = false;
-
-        if( mScrollTimer->isActive() )
-            mScrollTimer->stop();
-
-        // was only click inside selection, nothing dragged?
-        if( mDragStartPossible )
-        {
-            selectAll( false );
-            mDragStartTimer->stop();
-            mDragStartPossible = false;
-
-            placeCursor( mDragStartPoint );
-            ensureCursorVisible();
-
-            unpauseCursor();
-        }
-        // was end of selection operation?
-        else if( mTableRanges->hasSelection() )
-        {
-            if( QApplication::clipboard()->supportsSelection() )
-            {
-                mClipboardMode = QClipboard::Selection;
-                q->disconnect( QApplication::clipboard(), SIGNAL(selectionChanged()) );
-
-                copy();
-
-                //TODO: why did we do this? And why does the disconnect above not work?
-                // got connected multiple times after a few selections by mouse
-        //         connect( QApplication::clipboard(), SIGNAL(selectionChanged()), SLOT(clipboardChanged()) );
-                mClipboardMode = QClipboard::Clipboard;
-            }
-        }
-    }
-    // middle mouse button paste?
-    else if( mouseEvent->button() == Qt::MidButton && !isEffectiveReadOnly() )
-    {
-        pauseCursor();
-        mValueEditor->finishEdit();
-
-        placeCursor( releasePoint );
-
-        // replace no selection?
-        if( mTableRanges->hasSelection() && !mTableRanges->selectionIncludes(mTableCursor->index()) )
-            mTableRanges->removeSelection();
-
-        mClipboardMode = QClipboard::Selection;
-        paste();
-        mClipboardMode = QClipboard::Clipboard;
-
-        // ensure selection changes to be drawn TODO: create a insert/pasteAtCursor that leaves out drawing
-        updateChanged();
-
-        ensureCursorVisible();
-        unpauseCursor();
-    }
-
-    emit q->cursorPositionChanged( cursorPosition() );
-
-    mInDoubleClick = false;
-
-    if( mTableRanges->selectionJustStarted() )
-        mTableRanges->removeSelection();
-
-    const bool newHasSelection = mTableRanges->hasSelection();
-    emit q->selectionChanged( mTableRanges->selection() );
-    if( oldHasSelection != newHasSelection )
-    {
-        if( !mOverWrite ) emit q->cutAvailable( newHasSelection );
-        emit q->copyAvailable( newHasSelection );
-        emit q->hasSelectedDataChanged( newHasSelection );
-    }
-}
-
-
-// gets called after press and release instead of a plain press event (?)
-void ByteArrayRowViewPrivate::mouseDoubleClickEvent( QMouseEvent* mouseEvent )
-{
-    Q_Q( ByteArrayRowView );
-
-    // we are only interested in LMB doubleclicks
-    if( mouseEvent->button() != Qt::LeftButton )
-    {
-        mouseEvent->ignore();
-        return;
-    }
-
-    mDoubleClickLine = mTableCursor->line();
-
-    const Address index = mTableCursor->validIndex();
-
-    if( mActiveCoding == AbstractByteArrayView::CharCodingId )
-    {
-        selectWord( index );
-
-        // as we already have a doubleclick maybe it is a tripple click
-        mTrippleClickTimer->start( qApp->doubleClickInterval() );
-        mDoubleClickPoint = mouseEvent->globalPos();
-    }
-    //  else
-    //    mValueEditor->goInsideByte(); TODO: make this possible again
-
-    mInDoubleClick = true; //
-    mMousePressed = true;
-
-    emit q->doubleClicked( index );
-}
-
-
-void ByteArrayRowViewPrivate::autoScrollTimerDone()
-{
-    Q_Q( ByteArrayRowView );
-
-    if( mMousePressed )
-        handleMouseMove( q->viewportToColumns(q->viewport()->mapFromGlobal( QCursor::pos() )) );
-}
-
-
-void ByteArrayRowViewPrivate::handleMouseMove( const QPoint& point ) // handles the move of the mouse with pressed buttons
-{
-    Q_Q( ByteArrayRowView );
-
-    const bool oldHasSelection = mTableRanges->hasSelection();
-    const int yOffset = q->yOffset();
-    const int behindLastYOffset = yOffset + q->visibleHeight();
-    // scrolltimer but inside of viewport?
-    if( mScrollTimer->isActive() )
-    {
-        if( yOffset <= point.y() && point.y() < behindLastYOffset )
-            mScrollTimer->stop();
-    }
-    // no scrolltimer and outside of viewport?
-    else
-    {
-        if( point.y() < yOffset || behindLastYOffset <= point.y() )
-            mScrollTimer->start( DefaultScrollTimerPeriod );
-    }
-    pauseCursor();
-
-    placeCursor( point );
-    ensureCursorVisible();
-
-    // do wordwise selection?
-    if( mInDoubleClick && mTableRanges->hasFirstWordSelection() )
-    {
-        Address newIndex = mTableCursor->realIndex();
-        const AddressRange firstWordSelection = mTableRanges->firstWordSelection();
-        const WordByteArrayService WBS( mByteArrayModel, charCodec() );
-        // are we before the selection?
-        if( firstWordSelection.startsBehind(newIndex) )
-        {
-            mTableRanges->ensureWordSelectionForward( false );
-            newIndex = WBS.indexOfLeftWordSelect( newIndex );
-        }
-        // or behind?
-        else if( firstWordSelection.endsBefore(newIndex) )
-        {
-            mTableRanges->ensureWordSelectionForward( true );
-            newIndex = WBS.indexOfRightWordSelect( newIndex );
-        }
-        // or inside?
-        else
-        {
-            mTableRanges->ensureWordSelectionForward( true );
-            newIndex = firstWordSelection.nextBehindEnd();
-        }
-
-        mTableCursor->gotoIndex( newIndex );
-    }
-
-    if( mTableRanges->selectionStarted() )
-        mTableRanges->setSelectionEnd( cursorPosition() );
-
-    updateChanged();
-
-    unpauseCursor();
-
-    const bool newHasSelection = mTableRanges->hasSelection();
-    emit q->cursorPositionChanged( cursorPosition() );
-    emit q->selectionChanged( mTableRanges->selection() );
-    if( oldHasSelection != newHasSelection )
-    {
-        if( !mOverWrite ) emit q->cutAvailable( newHasSelection );
-        emit q->copyAvailable( newHasSelection );
-        emit q->hasSelectedDataChanged( newHasSelection );
-    }
-}
-
-
-void ByteArrayRowViewPrivate::startDrag()
-{
-    Q_Q( ByteArrayRowView );
-
-    // reset states
-    mMousePressed = false;
-    mInDoubleClick = false;
-    mDragStartPossible = false;
-
-    // create data
-    QMimeData *dragData = selectionAsMimeData();
-    if( !dragData )
-        return;
-
-    QDrag *drag = new QDrag( q );
-    drag->setMimeData( dragData );
-
-    Qt::DropActions request = (isEffectiveReadOnly()||mOverWrite) ? Qt::CopyAction : Qt::CopyAction|Qt::MoveAction;
-    Qt::DropAction dropAction = drag->exec( request );
-
-    if( dropAction == Qt::MoveAction )
-    {
-        AbstractByteArrayView* targetByteArrayView = qobject_cast<AbstractByteArrayView*>( drag->target() );
-        // Not inside this widget itself?
-        if( ! targetByteArrayView
-            || targetByteArrayView->byteArrayModel() != q->byteArrayModel() )
-            removeSelectedData();
-    }
-}
-
 ByteArrayRowViewPrivate::~ByteArrayRowViewPrivate()
 {
-    delete mCursorPixmaps;
-    delete mStylist;
 }
 
 }
